!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


MODULE globals
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Q,G,F,source,Jx,Jy,YY,hi,Grav
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: p,rho,u,v,T,e,ss,ie,Rgas,gamma
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: tauxx,tauxy,tauyy,Tx,Ty,dQx,dQy
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: y,x,diff_f
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: mu,beta,kappa
DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: dudx,dudy,dvdx,dvdy
DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: diff
DOUBLE PRECISION, PARAMETER :: half = 0.5D0, one = 1.0D0, two = 2.0D0,three = 3.0D0, zero = 0.0D0, pi = 3.14159265359
DOUBLE PRECISION, PARAMETER :: R = 8.314472D7
DOUBLE PRECISION :: IL,JL,dt,t_init,time,time_l,time_v
DOUBLE PRECISION :: Mw,dr
DOUBLE PRECISION :: mu_f,beta_f,kappa_f
DOUBLE PRECISION :: tp, velxbc, velybc
INTEGER, PARAMETER :: flen=90,inputUnit=12,ioUnit=11
INTEGER :: i,j,k,cheap,filter_lim
INTEGER :: bx1,bxn,by1,byn,counter
CHARACTER(LEN=flen) :: prob,jobname
character(LEN=5) :: stab
CHARACTER(LEN=flen) :: inputFile
END MODULE globals

MODULE inputs
IMPLICIT NONE
INTEGER :: nx=100
INTEGER :: ny=100
INTEGER :: periodx=1
INTEGER :: periody=1
INTEGER :: ns=1
INTEGER :: tt=1000
INTEGER :: restart_log=0
DOUBLE PRECISION :: dx=1.0d-2
DOUBLE PRECISION :: dy=1.0d-2
DOUBLE PRECISION :: cfl=1.0d0
DOUBLE PRECISION :: simtime=1
DOUBLE PRECISION :: viztime=1.d-1
DOUBLE PRECISION :: gamma_f(2)=1.4d0

DOUBLE PRECISION :: P0 = 1.0d5
DOUBLE PRECISION :: T0 = 300.0d0
DOUBLE PRECISION :: Gx = 0.0d0
DOUBLE PRECISION :: Gy = 0.0d0
DOUBLE PRECISION :: x1 = 0.0d0
DOUBLE PRECISION :: y1 = 0.0d0

INTEGER :: nsmax = 2
DOUBLE PRECISION :: molwt(2) = 1.0d0 
LOGICAL :: sing=.FALSE.

END MODULE inputs

!MODULE jacobian

!IMPLICIT NONE
!DOUBLE PRECISION, DIMENSION(4,4) :: Ap,An,Bp,Bn
!DOUBLE PRECISION, DIMENSION(4) :: Qhat

!END MODULE jacobian

!MODULE AFLES_vars

!IMPLICIT NONE

!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: mu,beta,kappa,mu_s, beta_s,kappa_s
!DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: diff,diff_s
!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: beta_1, beta_2
!DOUBLE PRECISION :: del

!END MODULE AFLES_vars


!MODULE metrics
!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: x_c,y_c,dxdE,dxdN,dydE,dydN
!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: dEdx,dNdx,dEdy,dNdy
!DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: dxy, dQdE,dQdN,theta,del2
!DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: Gp,Fp,del_mu,del_beta
!DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: met_del,met,time_met
!DOUBLE PRECISION, DIMENSION(:,:,:,:), ALLOCATABLE :: x_del
!DOUBLE PRECISION, DIMENSION(:,:,:), ALLOCATABLE :: artEx,artNx,artEy,artNy
!DOUBLE PRECISION :: dE, dN, del_min

!END MODULE metrics
