!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


    PROGRAM miranda_2d
    USE globals
    USE inputs
    IMPLICIT NONE
    NAMELIST /INPUT/ nx,ny,periodx,periody,ns,gamma_f,tt,restart_log, & 
         & dx,dy,cfl,simtime,viztime,molwt,Gx,Gy,x1,y1,sing, &
         & P0,T0
    INTEGER :: outflag
    


    !  Cheap Flag: 10= 10th order compact; 2 = 2nd order central, 
    cheap = 10
    
    filter_lim = 20

    IF (iargc() .EQ. 0) THEN
      inputFile='input'
    ELSE
      CALL getarg(1,inputFile)
    END IF

    OPEN(UNIT=11,FILE=TRIM(inputFile),FORM='FORMATTED',STATUS='OLD')
      READ(UNIT=11,NML=INPUT)
    CLOSE(11)

    CALL prob_inputs(inputFile)
    CALL prob_init 


    !	Set up output monitors and time
    time = t_init
    time_l = t_init
    time_v = t_init

    !	Set up the time integration and output initial conditions
    CALL courant
    CALL output

    WRITE (6, *) 'Initialization Sucessful'
    WRITE (6, *) 'Writing initial vizfile ... ' 
    WRITE (6, *) '-------------------- SIMULATION STARTED ------------------------'
    WRITE (6, *) '|  Time Step #  |  Physical Time  |  dt Limit |  dt Value      |'

    !	Main Loop for the solver
    DO k = 1, tt
	CALL get_dt(counter,outflag)
	
        !  Keep track of time
    	WRITE (6, '(A,I6.6,A,ES12.6,A,A,A,ES12.6,A)') ' |    ',k,'     |  ' &
          ,(time),'   |   ',stab,'   |  ', (dt),'  |'
    	if (time >= simtime) then
          CALL output
          exit
    	end if

        !  Integrate PDE one time step
    	CALL rk4
        time_l = time
        time = dt + time

	!	Logic to output viz files for every specified time period	
    	if (outflag .eq. 1) then 
    	    CALL output
            WRITE (6, *) 'Writing vizfile number...',counter
            WRITE (6, *) '|  Time Step #  |  Physical Time  |  dt Limit |  dt Value      |'
            time_v = time
        end if
	
    END DO

END PROGRAM miranda_2d









