!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.

! Coefficients for band-diagonal matrix solver (Ax=By)
 MODULE matrices
  INTEGER, PARAMETER :: nlu = 9
  ! Tenth-order compact first-derivative with conservative 3rd-order boundaries----------------------------------------------------
  INTEGER, PARAMETER :: A10dm1 = 2, A10dm2 = 2
  INTEGER, PARAMETER :: A10dcol = 7
  INTEGER, PARAMETER :: A10drow = 5
  INTEGER, PARAMETER :: B10dm1 = 3, B10dm2 = 3
  INTEGER, PARAMETER :: B10dcol = 8
  INTEGER, PARAMETER :: B10drow = A10drow
  INTEGER, PARAMETER :: diag10d = A10dm1+1+A10dm2
  LOGICAL, PARAMETER :: implicit_d1 = .true.
  DOUBLE PRECISION, DIMENSION(A10dcol,A10drow) :: A10d
  DOUBLE PRECISION, DIMENSION(B10dcol,B10drow) :: B10d
  DATA A10d / &
         4.725D0       , 9.45D0        , 0.0D0         , 0.0D0         , 0.0D0        , 0.0D0         , 0.0D0         , &
         1.94578125D0  , 7.783125D0    , 1.94578125D0  , 0.0D0         , 0.0D0        , 0.0D0         , 0.0D0         , &
         0.2964375D0   , 4.743D0       , 10.67175D0    , 4.743D0       , 0.2964375D0  , 0.0D0         , 0.0D0         , &
         0.0D0         , 0.451390625D0 , 4.63271875D0  , 9.38146875D0  , 4.63271875D0 , 0.451390625D0 , 0.0D0         , &
         0.0D0         , 0.0D0         , 0.45D0        , 4.5D0         , 9.0D0         , 4.5D0        , 0.45D0        /
  DATA B10d / &
         -11.8125D0     ,  9.45D0    , 2.3625D0      , 0.0D0      , 0.0D0         , 0.0D0      , 0.0D0      , 0.0D0         , &
         -5.83734375D0  ,  0.0D0     , 5.83734375D0  , 0.0D0      , 0.0D0         , 0.0D0      , 0.0D0      , 0.0D0         , &
         -1.23515625D0  , -7.905D0   , 0.0D0         , 7.905D0    , 1.23515625D0  , 0.0D0      , 0.0D0      , 0.0D0         , &
         -0.015D0       , -1.53D0    , -6.66984375D0 , 0.0D0      , 6.66984375D0  , 1.53D0     , 0.015D0    , 0.0D0         , &
         0.0D0          , -0.015D0   , -1.515D0      , -6.375D0   , 0.0D0         , 6.375D0    , 1.515D0    , 0.015D0       /
  INTEGER, ALLOCATABLE, DIMENSION(:) :: indx10dx,indx10dy,indx10dz
  DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: a10dx,a10dy,a10dz,al10dx,al10dy,al10dz,alu10dx,alu10dy,alu10dz
  DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: dxbw1, dxbwn, dybw1, dybwn, dzbw1, dzbwn

  ! Tenth-order compact second-derivative-------------------------------------------------------------------------------------------
  INTEGER, PARAMETER :: Ad2m1 = 2, Ad2m2 = 2
  INTEGER, PARAMETER :: Ad2col = 6
  INTEGER, PARAMETER :: Ad2row = 4
  INTEGER, PARAMETER :: Bd2m1 = 3, Bd2m2 = 3
  INTEGER, PARAMETER :: Bd2col = 7
  INTEGER, PARAMETER :: Bd2row = Ad2row
  INTEGER, PARAMETER :: diagd2 = Ad2m1+1+Ad2m2
  LOGICAL, PARAMETER :: implicit_d2 = .true.
  DOUBLE PRECISION, DIMENSION(Ad2col,Ad2row) :: Ad2
  DOUBLE PRECISION, DIMENSION(Bd2col,Bd2row) :: Bd2
  DATA Ad2 / &
         1.0D0  , 11.0D0  , 0.0D0    , 0.0D0     , 0.0D0    , 0.0D0   , &
         1.0D0  , 10.0D0  , 1.0D0    , 0.0D0     , 0.0D0    , 0.0D0   , &
         23.0D0 , 688.0D0 , 2358.0D0 , 688.0D0   , 23.0D0   , 0.0D0   , &
         0.0D0  , 387.0D0 , 6012.0D0 , 16182.0D0 , 6012.0D0 , 387.0D0 /
  DATA Bd2 / &
         13.0D0  , -27.0D0  , 15.0D0    , -1.0D0     , 0.0D0    , 0.0D0    , 0.0D0  , &
         12.0D0  , -24.0D0  , 12.0D0    , 0.0D0      , 0.0D0    , 0.0D0    , 0.0D0  , &
         465.0D0 , 1920.0D0 , -4770.0D0 , 1920.0D0   , 465.0D0  , 0.0D0    , 0.0D0  , &
         79.0D0  , 4671.0D0 , 9585.0D0  , -28670.0D0 , 9585.0D0 , 4671.0D0 , 79.0D0 /
  INTEGER, ALLOCATABLE, DIMENSION(:) :: indxd2x,indxd2y,indxd2z
  DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: ad2x,ad2y,ad2z,ald2x,ald2y,ald2z,alud2x,alud2y,alud2z

  ! Filters-------------------------------------------------------------------------------------------------------------------------
  INTEGER, PARAMETER :: nfilters=7, nfsharp=5
  INTEGER, PARAMETER :: A8fm1 = 2, A8fm2 = 2 
  INTEGER, PARAMETER :: A8fcol = 7
  INTEGER, PARAMETER :: A8frow = 5
  INTEGER, PARAMETER :: B8fm1 = 4, B8fm2 = 4 
  INTEGER, PARAMETER :: B8fcol = 9
  INTEGER, PARAMETER :: B8frow = A8frow
  INTEGER, PARAMETER :: diag8f = A8fm1+1+A8fm2
  LOGICAL, PARAMETER, DIMENSION(nfilters) :: implicit_f = (/ .true., .true., .true., .true., .true., .false., .false. /)
  DOUBLE PRECISION, DIMENSION(A8fcol,A8frow) :: A8f 
  DOUBLE PRECISION, DIMENSION(B8fcol,B8frow) :: B8f
  ! 9/10 truncation-----------------------------------------------------------------------------------------------------------------
  DATA A8f / &
       &  1.0D0     , 0.0D0      , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , &
       &  4.997D-1  , 1.0D0      , 4.997D-1  , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , &
       &  1.6688D-1 , 6.6624D-1  , 1.0D0     , 6.6624D-1 , 1.6688D-1 , 0.0D0     , 0.0D0     , &
       &  0.0D0     , 1.6688D-1  , 6.6624D-1 , 1.0D0     , 6.6624D-1 , 1.6688D-1 , 0.0D0     , &
       &  0.0D0     , 0.0D0      , 1.6688D-1 , 6.6624D-1 , 1.0D0     , 6.6624D-1 , 1.6688D-1 /
  DATA B8f / &
       &  1.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0  , 0.0D0   , &
       &  4.9985D-1 , 9.997D-1  , 4.9985D-1 , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0     , 0.0D0  , 0.0D0   , &
       &  1.668D-1  , 6.6656D-1 , 9.9952D-1 , 6.6656D-1 , 1.668D-1  , 0.0D0     , 0.0D0     , 0.0D0  , 0.0D0   , &
       &  4.0D-5    , 1.6672D-1 , 6.6652D-1 , 9.9968D-1 , 6.6652D-1 , 1.6672D-1 , 4.0D-5    , 0.0D0  , 0.0D0   , &
       &  -5.0D-6   , 4.0D-5    , 1.6674D-1 , 6.6652D-1 , 9.9965D-1 , 6.6652D-1 , 1.6674D-1 , 4.0D-5 , -5.0D-6 /


 END MODULE matrices
