!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


SUBROUTINE metric_terms
USE globals
USE inputs
USE metrics
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(nx,ny) :: tmp,tmp1
DOUBLE PRECISION, DIMENSION(nx) :: dumx
DOUBLE PRECISION, DIMENSION(ny) :: dumy
INTEGER :: n,Nb


!  Set transformed grid size here (set to unity by default)
dE = dx
dN = dy

!  Allocate the arrays 
ALLOCATE(x_c(nx,ny))
ALLOCATE(y_c(nx,ny))
ALLOCATE(dxdE(nx,ny))
ALLOCATE(dxdN(nx,ny))
ALLOCATE(dydE(nx,ny))
ALLOCATE(dydN(nx,ny))
ALLOCATE(dEdx(nx,ny))
ALLOCATE(dNdx(nx,ny))
ALLOCATE(dEdy(nx,ny))
ALLOCATE(dNdy(nx,ny))
ALLOCATE(dxy(nx,ny))
ALLOCATE(dQdE(nx,ny))
ALLOCATE(dQdN(nx,ny))
ALLOCATE(Gp(nx,ny,4+(ns-1)))
ALLOCATE(Fp(nx,ny,4+(ns-1)))
ALLOCATE(met_del(nx,ny,2))
ALLOCATE(time_met(nx,ny,6))
ALLOCATE(met(nx,ny,2))
ALLOCATE(theta(nx,ny))
ALLOCATE(del2(nx,ny))
ALLOCATE(del_mu(nx,ny,2))
ALLOCATE(del_beta(nx,ny,2))
ALLOCATE(x_del(nx,ny,2,2))
!  Directional bulk viscosity terms
ALLOCATE(artEx(nx,ny,4+(ns-1)))
ALLOCATE(artNx(nx,ny,4+(ns-1)))
ALLOCATE(artEy(nx,ny,4+(ns-1)))
ALLOCATE(artNy(nx,ny,4+(ns-1)))


!  Fill (x,y) location for every grid point for the derivative routines
!  This is default orthogonal cartesian
CALL grid_gen
DO i = 1 , nx
	DO j = 1 , ny
		x_c(i,j) = x(i)
		y_c(i,j) = y(j)
	END DO
END DO

!  Prob dependent grids called in prob_grid
CALL prob_grid

!  Get the derivative terms using 10th order compact scheme
CALL tenth_order(x_c,dxdE,1,nx,ny,dE,0)
CALL tenth_order(x_c,dxdN,2,nx,ny,dN,0)
CALL tenth_order(y_c,dydE,1,nx,ny,dE,0)
CALL tenth_order(y_c,dydN,2,nx,ny,dN,0)

!  Last needed metric term
IF(nx==1) THEN
  dxdE=one;dydE=one
END IF
IF(ny==1) THEN
  dxdN=one;dydN=one
END IF
dxy = dxdE*dydN - dxdN*dydE


dNdy = dxdE/dxy
dNdx = -dydE/dxy
dEdx = dydN/dxy
dEdy = -dxdN/dxy


x_del(:,:,1,1) = dxdE*dE
x_del(:,:,1,2) = dydE*dE
x_del(:,:,2,1) = dxdN*dN
x_del(:,:,2,2) = dydN*dN
 
!met_del(:,:,1) = sqrt( (dE*dxdE)**two + (dN*dxdN)**two)
!met_del(:,:,2) = sqrt( (dE*dydE)**two + (dN*dydN)**two) 

met_del(:,:,1) = sqrt( (dE*dxdE)**two + (dE*dydE)**two)
met_del(:,:,2) = sqrt( (dN*dxdN)**two + (dN*dydN)**two)

met(:,:,1) = sqrt( (dEdx)**two + (dEdy)**two)
met(:,:,2) = sqrt( (dNdx)**two + (dNdy)**two)

!  Metric length terms for CFL time step calculation
tmp =  sqrt( (dxdE*dE)**2 + (dydE*dE)**2 )
tmp1 = sqrt( (dxdN*dN)**2 + (dydN*dN)**2 )
time_met(:,:,1) = dxdE*dE/tmp
time_met(:,:,2) = dydE*dE/tmp
time_met(:,:,3) = dxdN*dN/tmp1
time_met(:,:,4) = dydN*dN/tmp1
time_met(:,:,5) = tmp
time_met(:,:,6) = tmp1

!del2 = sqrt( tmp**2 + tmp1**2 )
del2 = min(tmp,tmp1)

del_min = minval(dydE**2 + dxdE**2 + dydN**2 + dxdN**2 )
del_min = sqrt(del_min)
!print*,del_min



END SUBROUTINE metric_terms




